// FileManager.h

#ifndef _FILEMANAGER_H_
#define _FILEMANAGER_H_

#include "afx.h"

#include "refcount.h"
#include "smartpointer.h"

#include <string>
#include <vector>
#include <stack>
#include <set>
#include <map>

class FileManager;
class FileAssociationManager;

FileManager & getFileManager();

//typedef void* HANDLE;
typedef unsigned char byte;

//------------------------------------------------------------------------------
// subroutines
//------------------------------------------------------------------------------

bool AppendSlash ( std::string & path );
bool ConcatPathFile ( const char * path, const char * file, std::string & path_file );
bool GetFileFromPath ( const char * path, std::string & file );
bool GetDirectoryFromPath ( const char * path, std::string & directory );
bool GetRelativePath ( const char * path, const char * base_path, std::string & relative_path );
bool GetFullPathFromRelative ( const char * relative_path, std::string & full_path );
bool EvalPath ( const char * source_path, const char * input_base, const char * output_base, std::string & target_path );

//------------------------------------------------------------------------------
// File
//------------------------------------------------------------------------------

class File : public RefCountBase
{
public:
	enum E_access_mode { ac_read, ac_write, ac_read_write, ac_create };

public:
	class constructor_exception
	{
	public:
		constructor_exception ( const std::string & error )
			: error_ ( error )
		{}

		std::string error_;
	};

public:
	File ( const char * file_path );
	~File();
	bool open ( E_access_mode open_mode );
	void close();
	bool read ( void * buffer, size_t bytes, size_t & readed );
	bool read ( void * buffer, size_t bytes );
	bool write ( const void * buffer, size_t bytes, size_t & written );
	bool write ( const void * buffer, size_t bytes );
	bool seekToEndOfFile();
	bool getLength ( size_t & length );
	bool getName ( std::string & file_name );
	bool getFullPath ( std::string & file_path );
	bool getDirectory ( std::string & file_path );
	bool fileExistAndNotDirectory() const;

private:
	HANDLE handle_;
	std::string file_;
	std::string full_path_;
};

typedef RefCountBasePtr<File> PFile;

//void setError ( PFile file, const std::string & suffix );

//------------------------------------------------------------------------------

namespace FileNamespace
{

//------------------------------------------------------------------------------
// FileIteratorWithPred
//------------------------------------------------------------------------------

typedef bool (*file_pred_fun) ( const WIN32_FIND_DATA & findFileData );

class Win32FileIteratorWithPred : public RefCountBase
{
public:
	Win32FileIteratorWithPred
	(
		const char * file_mask,
		file_pred_fun fun
	);
	~Win32FileIteratorWithPred();
	bool next ( WIN32_FIND_DATA & findFileData );

private:
	std::string file_mask_;
	file_pred_fun fun_;
	HANDLE handle_;
	bool bEnd_;
};

typedef RefCountBasePtr<Win32FileIteratorWithPred> PWin32FileIteratorWithPred;

//------------------------------------------------------------------------------

bool isFile ( const WIN32_FIND_DATA & findFileData );
bool isDirectory ( const WIN32_FIND_DATA & findFileData );
bool isSubDirectory ( const WIN32_FIND_DATA & findFileData );
bool getCurrentDirectory ( std::string & s );

//------------------------------------------------------------------------------

} // namespace FileNamespace

//------------------------------------------------------------------------------
// FileIterator
//------------------------------------------------------------------------------

class FileIterator : public RefCountBase
{
public:
	FileIterator();
	virtual ~FileIterator();
	virtual PFile next() = 0;
};

typedef RefCountBasePtr<FileIterator> PFileIterator;

//------------------------------------------------------------------------------
// FileSearchIterator
//------------------------------------------------------------------------------

using namespace FileNamespace;

class FileSearchIterator : public FileIterator
{
public:
	FileSearchIterator
	(
		const char * search_path,
		const char * file_mask,
		bool recursive
	);
	virtual ~FileSearchIterator();
	virtual PFile next();

private:
	bool setNextSearchDirectory();

private:
	std::string search_path_;
	std::string file_mask_;
	bool recursive_;

	std::string old_current_directory_; // for restore search directory
	bool bEnd_;
	PWin32FileIteratorWithPred file_iter_;
	bool bFileIterator_;
	std::stack<PWin32FileIteratorWithPred> directory_stack_;
	std::stack<std::string> current_directory_stack_;
	bool bDirectoryIterator_;
	int directory_;
	BOOL bCurrentDirectory_;
};

typedef RefCountBasePtr<FileSearchIterator> PFileSearchIterator;

//------------------------------------------------------------------------------
// DirectoryIterator
//------------------------------------------------------------------------------

class DirectoryIterator : public FileIterator
{
public:
	DirectoryIterator
	(
		const char * search_path
	);
	virtual ~DirectoryIterator();
	virtual PFile next();

private:
	std::string search_path_;
	std::string old_current_directory_; // for restore search directory
	bool bEnd_;
	PWin32FileIteratorWithPred directory_iter_;
	BOOL bCurrentDirectory_;
};

typedef RefCountBasePtr<DirectoryIterator> PDirectoryIterator;

//------------------------------------------------------------------------------
// FileManager
//------------------------------------------------------------------------------

class FileManager
{
public:
	PFile getFile ( const char * file_path );
	PFile getFile ( const char * directory, const char * relative_path );
	PFileIterator createFileIterator
		( const char * file_path, const char * file, bool recursive );

	static bool getCurrentDirectory ( std::string & current_directory );
	static bool setCurrentDirectory ( const std::string & current_directory );
	static bool normalizePath ( std::string & current_directory );
	static bool isDirectoryExist ( const char * path );
	static bool createDirectory ( const char * path );
	static bool createDirectoryRecursive ( const char * path );

// singleton support

public:
	static FileManager & getInstance();

private:
	FileManager();
	FileManager ( const FileManager & rhs );
	FileManager& operator= ( const FileManager & rhs );
	~FileManager();
};

//------------------------------------------------------------------------------

#endif // __FILEMANAGER_H__
